'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetch = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

require('../fetch/fetch-polyfill');

var _interceptor = require('./interceptor');

var _interceptor2 = _interopRequireDefault(_interceptor);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _transformResponse = require('../util/transformResponse');

var _transformResponse2 = _interopRequireDefault(_transformResponse);

var _checkStatus = require('../util/checkStatus');

var _checkStatus2 = _interopRequireDefault(_checkStatus);

var _timeout = require('./timeout');

var _timeout2 = _interopRequireDefault(_timeout);

var _promiseTask = require('./promiseTask');

var _promiseTask2 = _interopRequireDefault(_promiseTask);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var bodyMethods = ['POST', 'PUT', 'PATCH'];
var noBodyMethods = ['GET', 'DELETE', 'OPTIONS', 'HEAD'];
var fetch = self.fetch;

var zyFetch = function zyFetch(init, option) {
  option = option || {};
  var config = {};
  // merge config
  (0, _assign2.default)(config, zyFetch.config, option);

  var request = new Request(init, config);

  var promiseTask = new _promiseTask2.default();

  function _init() {
    // set request promise
    zyFetch.interceptors.request.forEach(function (interceptor) {
      promiseTask.add(interceptor.onFulfilled, interceptor.onRejected);
    });
    //set timeout
    if (config.timeout) {
      promiseTask.add((0, _timeout2.default)(config.timeout));
    } else {
      promiseTask.add(fetch);
    }

    //set checkStatus promise
    promiseTask.add(_checkStatus2.default);

    //set before transform response interceptors promise
    zyFetch.interceptors.response.noTransform.forEach(function (interceptor) {
      promiseTask.add(interceptor.onFulfilled, interceptor.onRejected);
    });

    //set transform response promise
    if (config.transformResponse) {
      promiseTask.add(_transformResponse2.default.bind(this, config.responseType));
      // set after transform response promise
      zyFetch.interceptors.response.transform.forEach(function (interceptor) {
        promiseTask.add(interceptor.onFulfilled, interceptor.onRejected);
      });
    }
  }

  _init();
  return promiseTask.execute(request);
};

zyFetch.interceptors = {
  request: new _interceptor2.default(),
  response: {
    transform: new _interceptor2.default(),
    noTransform: new _interceptor2.default()
  }
};

bodyMethods.forEach(function (method) {
  zyFetch[method] = function (init, body, option) {
    (0, _assign2.default)(option, {
      method: method,
      body: body
    });
    return zyFetch(init, option);
  };
});

noBodyMethods.forEach(function (method) {
  zyFetch[method] = function (init, option) {
    (0, _assign2.default)(option, {
      method: method
    });
    return zyFetch(init, option);
  };
});

/**
 *
 * @param fetchs some fetch
 * @returns {Promise.<*[]>} return promise
 */
zyFetch.all = function (fetchs) {
  return _promise2.default.all(fetchs);
};

/**
 * Execute fetch in sequence
 * @param fetchs some fetch
 */
zyFetch.allByOrder = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(fetchs) {
    var responseArray, i, response;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            responseArray = [];
            i = 0;

          case 2:
            if (!(i < fetchs.length)) {
              _context.next = 10;
              break;
            }

            _context.next = 5;
            return fetchs[i].call(this);

          case 5:
            response = _context.sent;

            responseArray.push(response);

          case 7:
            i++;
            _context.next = 2;
            break;

          case 10:
            return _context.abrupt('return', _promise2.default.resolve(responseArray));

          case 11:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * 将函数的数组参数解构 [1,2,3] => 1,2,3
 * @param cb
 * @returns {Function}
 */
zyFetch.spread = function (cb) {
  return function (args) {
    cb.apply(null, args);
  };
};

zyFetch.config = _config2.default;
zyFetch.polyfill = fetch.polyfill;

exports.fetch = zyFetch;